"""『SS7』の「Op.Python実行」を実行するスクリプト

『SS7』で物件データを開き、入力CSVおよび結果CSVを出力し、それらのCSVファイルから物件情報を取得するためのものです。
指定されたバージョンと異なる場合には、データをコピーして変換した上で解析を実行し、
入力CSVと結果CSVを出力します。また、処理後に一時ファイルの削除も行います。

Copyright (C) 2024 UNION SYSTEM Inc.

"""

import os
import shutil
from typing import Union

from Python import Ss7Python as Cmd
import ss7_csv_input as csv_in
import ss7_csv_result as csv_re


def format_version(version_int: int) -> str:
    """バージョン番号をフォーマットします。

    Args:
        version_int (int): フォーマット前のバージョン番号。

    Returns:
        str: フォーマットされたバージョン番号。
    """
    # バージョン番号を文字列に変換
    version_str = str(version_int)

    # 文字列を所定のフォーマットに変換
    major_version = int(version_str[0])
    minor_version = int(version_str[1:3])
    patch_version = int(version_str[3:5])
    build_version = int(version_str[5:])

    formatted_version = (
        f"{major_version}.{minor_version}.{patch_version}.{build_version}"
    )

    return formatted_version


def get_version(
    ss7data_path: str,
    ss7_version: Union[str, None] = None,
) -> str:
    """
    物件データのバージョン情報を取得します。

    Args:
        ss7data_path (str): 物件データが保存されているフォルダパス
        ss7_version (Union[str, None]): 使用するSS7のバージョン。

    Returns:
        str: バージョン情報。物件でない場合は空文字列。
    """
    try:
        # Ss7Pythonコマンドの初期化
        Cmd.Init()
        Cmd.Start(ss7_version, 1)  # 1=ログファイルをクリアする

        # 物件バージョンの取得
        version_int: int = Cmd.GetInfoVersion(ss7data_path)
        if version_int == -1:
            print(f"{ss7data_path}: 物件ではない")
            return ""

        # バージョン番号を文字列に変換
        formatted_version: str = format_version(version_int)
        Cmd.End(2)
        return formatted_version

    except Exception as e:
        print(f"エラーが発生しました: {e}")
        return ""


def export_csv(
    ss7data_path: str,
    result_name: Union[str, None],
    input_csv_path: str,
    result_csv_path: str,
    ss7_version: Union[str, None] = None,
) -> bool:
    """
    SS7からCSVファイルにデータをエクスポートします。
    指定したバージョンと異なる場合は、コピーして変換して開き、準備計算まで行い入力csvと結果csvを出力します。
    コピーして変換して作成したデータはcsv出力後削除します。

    Args:
        ss7data_path (str): SS7データのパス。
        result_name (Union[str, None]): エクスポートする結果の名前。
        input_csv_path (str): 入力CSVファイルのパス。
        result_csv_path (str): 結果CSVファイルのパス。
        ss7_version (Union[str, None]): 使用するSS7のバージョン。

    Raises:
        Exception: データ処理中にエラーが発生した場合。
    """
    data = None
    err = None
    try:
        # SS7のセッションを開始し、ログファイルをクリアする
        Cmd.Init()
        Cmd.Start(ss7_version, 1)  # 1=ログファイルをクリアする

        # 物件データを開く（コピーして変換して開く）
        data = Cmd.Open(ss7data_path, 2, 1)

        # 最新のデータに変更
        data.Restore(result_name)

        # 入力データを取得
        input_data = data.GetInputData()

        # 入力CSVファイルに結果をエクスポート
        input_data.ExportInputCsv(input_csv_path, 1, 1)
        err = Cmd.GetLastError()
        if err.IsOK():
            print("入力csvの出力完了")
        else:
            errormessage = err.GetErrorMessage() if err else "詳細不明のエラー"
            print(f"エラーが発生しました: {errormessage}")
            return False

        # 準備計算まで解析が終了しているか？
        calc_item = "準備計算"
        calcend = data.IsCalcEnd(result_name, calc_item)
        if not calcend:
            # 準備計算が行われていない
            print("準備計算が行われていません。準備計算を行います。")
            data.Calculate(result_name, calc_item)

            # 再度、準備計算が終了しているか確認
            calcend = data.IsCalcEnd(result_name, calc_item)
            if calcend:
                print("準備計算が正常に終了しました。")
            else:
                print("準備計算が正常に終了しませんでした。処理を終了します。")
                return False

        # 解析結果を取得
        result_data = data.GetResultData(result_name)

        if calcend:
            export_items = "2.架構認識"
            # 断面算定まで解析が完了しているか
            calc_item = "断面算定"
            calcend = data.IsCalcEnd(result_name, calc_item)
            if calcend:
                export_items += ";10.ルート判定"

        # 結果CSVファイルに結果をエクスポート
        result_data.ExportResultCsv(export_items, result_csv_path, 1, 1, 1)

        err = Cmd.GetLastError()
        if err.IsOK():
            print("結果csvの出力完了")
            return True
        else:
            errormessage = err.GetErrorMessage() if err else "詳細不明のエラー"
            print(f"エラーが発生しました: {errormessage}")
            return False

    except Exception:
        # エラー情報を取得して表示
        err = Cmd.GetLastError()
        errormessage = err.GetErrorMessage() if err else "詳細不明のエラー"
        print(f"エラーが発生しました: {errormessage}")
        return False

    finally:
        if data:
            try:
                # 現在開いているフォルダの絶対パス
                current_path = data.GetPathName()
                data.Close(2)  # 2=保存しない
            except Exception as close_error:
                print(f"データを閉じる際にエラーが発生しました: {close_error}")
            finally:
                Cmd.End(2)  # 2=保存しない
                cleanup_temporary_files(ss7data_path, current_path)


def cleanup_temporary_files(ss7data_path: str, current_path: str) -> None:
    """
    一時ファイルのクリーンアップ

    Args:
        ss7data_path (str): 元のSS7データのパス。
        current_path (str): 現在の処理中のデータのパス。
    """
    if ss7data_path != current_path:
        print("コピーした物件を削除します。")
        if os.path.isdir(current_path):
            shutil.rmtree(current_path)
        elif os.path.isfile(current_path):
            os.remove(current_path)


# 構造種別
STRUCTURE_TYPES = {
    2: "RC",
    3: "SRC",
    4: "S",
    5: "CFT",
    7: "木",
    8: "木+RC",
    10: "木+S",
}
# 計算ルート
ROUTE_DICT = {
    0: "なし",
    1: "ルート1-1",
    2: "ルート1-2",
    7: "ルート1-3",
    3: "ルート2-1",
    4: "ルート2-2",
    5: "ルート2-3",
    6: "ルート3",
}

# 基礎の考慮
FOUNDATION_CONSIDERATIONS = {1: "しない", 2: "する", 3: "応力直接入力"}

# 基礎形式
KISO_SUPPORT = {1: "直接基礎", 2: "場所打ち杭 基礎", 3: "既製杭 基礎"}

# 基礎による応力解析モデル
FOUNDATION_MODEL = {
    1: "上部と基礎梁への影響を無視",
    2: "上部下部分離モデル",
    4: "上部下部一体モデル",
}


def get_input_info(input_csv_path: str) -> dict[str, Union[str, int, float]]:
    """
    入力CSVから情報を取得します。

    Args:
        input_csv_path (str): 入力CSVのパス。

    Returns:
        dict[str, Union[str, int, float]]: 入力情報を含む辞書。
    """
    input_info: dict[str, Union[str, int, float]] = {}

    # CSVファイルの読み込み
    input_csv = csv_in.Ss7CsvInput(input_csv_path)

    # 基本事項の取得
    basic_table = input_csv.search_table("基本事項")
    if basic_table:
        # 建物情報
        meisyou_list = ["工事名称", "略称", "日付", "担当者名"]
        for meisyo in meisyou_list:
            col = basic_table.search_col1(meisyo)
            data = basic_table.get_data(0, col)
            input_info[meisyo] = data or ""

        # 建物概要
        building_summary_list = [
            "X方向スパン数",
            "Y方向スパン数",
            "全階数",
            "地下階数",
            "PH階数",
        ]
        for summary in building_summary_list:
            col = basic_table.search_col2("建物概要", summary)
            data = basic_table.get_data(0, col)
            input_info[summary] = int(data) if data.isdigit() else 0

        # 主体構造
        main_structures_list = ["S", "RC", "SRC", "CFT", "木造"]
        for structure in main_structures_list:
            col = basic_table.search_col2("主体構造", structure)
            data = basic_table.get_data(0, col)
            input_info[structure] = "○" if data == "1" else ""

        # 構造種別、計算ルート
        calculation_route_list = ["構造種別", "X方向", "Y方向"]
        for route in calculation_route_list:
            col = basic_table.search_col2("計算ルート", route)
            data = basic_table.get_data(0, col)
            if route == "構造種別":
                input_info[f"{route}_計算ルート"] = STRUCTURE_TYPES.get(int(data), "")
            else:
                input_info[f"{route}_計算ルート"] = ROUTE_DICT.get(int(data), "")

        # 保有水平耐力
        horizontal_capacity_list = [
            "X方向正加力",
            "X方向負加力",
            "Y方向正加力",
            "Y方向負加力",
        ]
        for capacity in horizontal_capacity_list:
            col = basic_table.search_col2("保有水平耐力", capacity)
            data = basic_table.get_data(0, col)
            input_info[f"{capacity}_保有水平耐力"] = "○" if data == "1" else ""

    # 基礎計算条件の取得
    conditions_basic_table = input_csv.search_table_case("基礎計算条件", "基本事項")
    if conditions_basic_table:
        # 基礎の考慮
        col = conditions_basic_table.search_col1("基礎の考慮")
        data = conditions_basic_table.get_data(0, col)
        input_info["基礎の考慮"] = FOUNDATION_CONSIDERATIONS.get(int(data), "")

        if input_info["基礎の考慮"] == FOUNDATION_CONSIDERATIONS.get(2):

            # 基礎形式
            col = conditions_basic_table.search_col1("基礎形式")
            data = conditions_basic_table.get_data(0, col)
            input_info["基礎形式"] = KISO_SUPPORT.get(int(data), "") or ""

            if input_info["基礎形式"] == "直接基礎":
                # 基礎の種類
                kiso_list = [
                    "独立基礎",
                    "布基礎",
                    "べた基礎",
                ]
                for kiso in kiso_list:
                    col = conditions_basic_table.search_col3(
                        "基礎形式", "フーチング", kiso
                    )
                    data = conditions_basic_table.get_data(0, col)
                    input_info[kiso] = "○" if data == "1" else ""

            # 基礎による応力解析モデル
            col = conditions_basic_table.search_col1("基礎による応力解析モデル")
            data = conditions_basic_table.get_data(0, col)
            input_info["基礎による応力解析モデル"] = FOUNDATION_MODEL.get(int(data), "")

    return input_info


def get_result_info(
    result_csv_path: str,
    ikn_info: dict[str, Union[str, int, float]],
) -> dict[str, Union[str, int, float]]:
    """
    結果CSVから情報を取得します。

    Args:
        result_csv_path (str): 結果CSVのパス。
        ikn_info (dict[str, Union[str, int, float]]): 物件情報を含む辞書。

    Returns:
        dict[str, Union[str, int, float]]: 入力情報を含む辞書。
    """
    result_info: dict[str, Union[str, int, float]] = {
        "延べ床面積 (mm2)": "",
        "建物高さ (m)": "",
        "スパンの長さ (m)": "",
        "X加力_塔状比": "",
        "Y加力_塔状比": "",
        "X加力_Qu/Qun": "",
        "Y加力_Qu/Qun": "",
    }

    # CSVファイルの読み込み
    result_csv = csv_re.Ss7CsvResult(result_csv_path)

    # 延べ床面積の取得
    table = result_csv.search_table("構造階高")
    if table:
        data = 0.0
        col = table.search_col1("床面積")
        row_num = table.get_row_count()
        for row in range(row_num):
            cell_value = table.get(row, col).replace("*", "")
            if cell_value:  # セルの値が存在する場合のみ処理
                data += float(cell_value)
        result_info["延べ床面積 (mm2)"] = data

    # ルート判定表の取得
    # 主体構造の取得
    structure_type = ikn_info.get("構造種別_計算ルート", 0)
    route_table = result_csv.search_table(f"ルート判定表({structure_type})")
    if route_table:
        col_x = route_table.search_col1("判定値")
        col_y = route_table.search_col3("判定値", "ルート", "")
        route_list = ["建物高さ ≦ 31m", "スパンの長さ ≦ 6m"]

        for route in route_list:
            key_base = route.split(" ")[0]
            cell_value = route_table.get_data_key1(route, col_x).replace(" m", "")
            if cell_value:
                result_info[f"{key_base} (m)"] = float(cell_value)

        # 塔状比
        route_xy_list = ["塔状比 ≦ 4", "Qu/Qun ≧ 1.0"]
        for route_xy in route_xy_list:
            key_base = route_xy.split(" ")[0]
            cell_value_x = route_table.get_data_key1(route_xy, col_x).replace(" m", "")
            cell_value_y = route_table.get_data_key1(route_xy, col_y).replace(" m", "")

            if cell_value_x:
                result_info[f"X加力_{key_base}"] = float(cell_value_x)

            if cell_value_y:
                result_info[f"Y加力_{key_base}"] = float(cell_value_y)

    return result_info
