"""物件情報を取得するスクリプト

物件フォルダ、入力csv、結果csvから物件情報を取得します。

Copyright (C) 2024 UNION SYSTEM Inc.

"""

import os
import shutil
import re
import subprocess
from datetime import datetime
from typing import Union

import pandas as pd
from openpyxl.worksheet.worksheet import Worksheet
from openpyxl import load_workbook
from openpyxl.drawing.image import Image
from openpyxl.styles import Alignment

import ss7_calc


def get_latest_out_ikn_info(ikn_dir: str) -> tuple[str, str]:
    """
    指定されたディレクトリ内で、最も新しいCalcEndファイルを含む.outフォルダの名前とその更新日時を取得します。

    Args:
        ikn_dir (str): .iknフォルダのパス。

    Returns:
        tuple[str, str]: 最新のCalcEndファイルがあるフォルダ名とその更新日時。
    """
    # 最新CalcEndファイルの更新時間
    latest_update_time: float = 0.0
    # 最新のCalcEndファイルが含まれるフォルダ名
    latest_out_folder: str = ""
    # 最新のフォルダの更新日時（整形済み）
    latest_folder_update_time: str = ""

    # フォルダ名のパターンマッチング用正規表現
    pattern = re.compile(r"^結果\d+\.out$")

    # フォルダ内を探索
    for folder_name in os.listdir(ikn_dir):
        folder_path = os.path.join(ikn_dir, folder_name)

        # フォルダであり、名前が .out で終わるものを選択
        if os.path.isdir(folder_path) and pattern.match(folder_name):
            calcend_file_path = os.path.join(folder_path, "CalcEnd")

            # CalcEndファイルが存在するかチェック
            if os.path.exists(calcend_file_path):
                # CalcEndファイルの更新日時を取得
                calcend_update_time = os.path.getmtime(calcend_file_path)

                # 現在の最新ファイルよりも新しい場合、更新
                if calcend_update_time > latest_update_time:
                    latest_update_time = calcend_update_time
                    latest_out_folder = folder_name.replace(".out", "")

    # 最新のフォルダの更新日時を整形
    if latest_update_time != 0.0:
        latest_folder_update_time = datetime.fromtimestamp(latest_update_time).strftime(
            "%Y-%m-%d %H:%M:%S"
        )

    # 結果を整形して返す
    return (latest_out_folder, latest_folder_update_time)


def create_directories() -> str:
    """
    CSVの保存フォルダを作成し、パスを返します。

    Returns:
        str: CSVのフォルダのパス。
    """
    # スクリプトディレクトリから親ディレクトリを取得
    script_dir = os.path.dirname(os.path.abspath(__file__))
    parent_dir = os.path.dirname(script_dir)

    # 入力csvと結果csvのフォルダのパスを設定
    temp_csv_folder_path = os.path.join(parent_dir, "temp_csv")

    # 必要に応じてフォルダを作成
    os.makedirs(temp_csv_folder_path, exist_ok=True)

    return temp_csv_folder_path


def get_ikn_folder_info(
    base_path: str = r"C:\UsrData\Ss7Data",
    ss7_version: Union[str, None] = None,
) -> list[dict[str, Union[str, int, float]]]:
    """
    指定されたディレクトリ内の物件フォルダから物件情報を取得します。

    Args:
        base_path (str): ベースディレクトリのパス。
        ss7_version (str): 使用するSS7のバージョン。

    Returns:
        list[dict[str, Union[str, int, float]]]: 物件情報を含む辞書のリスト。
    """
    data_info: list[dict[str, Union[str, int, float]]] = []

    # 区切り文字の指定
    separator = "～" * 30

    # csvの保存フォルダを設定
    temp_csv_folder_path: str = create_directories()

    # 対象フォルダのリストを作成
    ikn_dirs: list = [
        os.path.join(root, dir_name)
        for root, dirs, _ in os.walk(base_path)
        for dir_name in dirs
        if dir_name.endswith(".ikn")  # ファイル名の末尾が「.ikn」であるもの
    ]

    # 対称フォルダの合計数
    total_folders: int = len(ikn_dirs)

    for i, ikn_dir in enumerate(ikn_dirs):
        # 物件情報を格納する辞書
        ikn_info: dict[str, Union[str, int, float]] = {}

        # 物件のフォルダ名
        ss7data_name: str = os.path.basename(ikn_dir)

        # 現在の処理状況を表示
        print(
            f"{separator}\n{i + 1}/{total_folders}\n{ss7data_name}の処理を行っています。\n"
        )

        # 最新の.outフォルダ情報を取得
        latest_folder, latest_time = get_latest_out_ikn_info(ikn_dir)

        # 物件のバージョンを取得
        data_version: str = ss7_calc.get_version(ikn_dir, ss7_version)

        # 情報を更新
        ikn_info.update(
            {
                "物件フォルダ名": ss7data_name.replace(".ikn", ""),
                "フォルダパス": ikn_dir,
                "バージョン": data_version,
                "最新結果": latest_folder or "不明",
                "更新日時": latest_time or "不明",
            }
        )

        # 入力csvと出力csvのパスの設定
        input_csv_path = os.path.join(temp_csv_folder_path, "input.csv")
        result_csv_path = os.path.join(temp_csv_folder_path, "result.csv")

        # 必要に応じて既存のファイルを削除
        for file_path in [input_csv_path, result_csv_path]:
            if os.path.exists(file_path):
                os.remove(file_path)

        # 入力CSVと結果CSVを出力
        if latest_folder:
            print("入力csvと結果csvを出力中...")
            if ikn_info["バージョン"] != ss7_version:
                print(
                    "物件のバージョンと実行中のバージョンが異なります。物件データをコピーして実行します。"
                )
            bexport: bool = ss7_calc.export_csv(
                ikn_dir,
                latest_folder,
                input_csv_path,
                result_csv_path,
                ss7_version,
            )

            if bexport:
                print("\nCSVからデータを取得中...")
                # 入力CSVからデータを取得
                try:
                    input_info = ss7_calc.get_input_info(input_csv_path)
                    ikn_info.update(input_info)
                except Exception as e:
                    print(f"入力データ取得中にエラーが発生しました: {e}")

                # 結果CSVからデータを取得
                try:
                    result_info = ss7_calc.get_result_info(result_csv_path, ikn_info)
                    ikn_info.update(result_info)
                except Exception as e:
                    print(f"結果データ取得中にエラーが発生しました: {e}")

                print("CSVからデータの取得完了")
        else:
            print("最新の結果フォルダが見つかりませんでした。")

        data_info.append(ikn_info)

    # csvの保存フォルダを削除
    shutil.rmtree(temp_csv_folder_path)

    return data_info


def output_directory(file_name: str) -> str:
    """
    指定されたファイル名を含む出力ディレクトリを作成し、そのパスを返します。

    Args:
        file_name (str): 作成するファイルの名前。

    Returns:
        str: 作成したファイルの完全なパス。
    """
    script_dir = os.path.dirname(os.path.abspath(__file__))
    parent_dir = os.path.dirname(script_dir)
    output_folder_path = os.path.join(parent_dir, "output")
    os.makedirs(output_folder_path, exist_ok=True)
    return os.path.join(output_folder_path, file_name)


def save_to_excel(
    data_info: list[dict[str, Union[str, int, float]]],
    excel_file_name: str = "folder_information.xlsx",
) -> None:
    """
    フォルダ情報をExcelにPandasを使用して保存します。

    Args:
        data_info (list[dict[str, Union[str, int, float]]]): 物件情報のリスト。
        excel_file_name (str): 保存するExcelファイルの名前。
    """
    # 区切り文字の指定
    separator: str = "～" * 30
    print(separator)
    print("Excelの出力")

    # 出力ファイルのパスを作成
    output_file_path = output_directory(excel_file_name)

    # 同名のファイルが存在する場合、_new, _new1, _new2 のようにファイル名を変更する
    base_name, file_extension = os.path.splitext(output_file_path)
    counter = 1

    while os.path.exists(output_file_path):
        # ファイル名に連番を付ける
        output_file_path = f"{base_name}_{counter}{file_extension}"
        counter += 1

    if counter >= 2:
        old_excel_file_name = excel_file_name
        excel_file_name = os.path.basename(output_file_path)
        print(
            f"既に「{old_excel_file_name}」が存在するため、ファイル名を「{excel_file_name}」に変更しました。\n"
        )

    try:
        # Pandas DataFrameを作成
        # 最も多くのキーを持つ辞書を探す
        largest_info = max(data_info, key=lambda x: len(x))
        # 画像のパスを追加する列をDataFrameに挿入
        columns = ["物件プレビュー"] + list(largest_info.keys())
        # Pandas DataFrameを作成
        df = pd.DataFrame(data_info, columns=columns)
        # Noの列を追加
        df.insert(0, "No", range(1, len(df) + 1))

        # ファイルが存在しない場合は新規作成
        with pd.ExcelWriter(output_file_path, engine="openpyxl") as writer:
            df.to_excel(writer, index=False, sheet_name="物件情報")

        # 画像やハイパーリンクを追加するためにopenpyxlでワークブックを再度読み込み
        wb = load_workbook(output_file_path)
        ws = wb.active
        start_row = 2

        # 画像の追加
        print("画像の追加")
        add_images(ws, data_info, start_row)

        # ハイパーリンクの設定
        print("ハイパーリンクの設定")
        headers = list(df.columns)
        set_hyperlink(ws, data_info, start_row, headers)

        # セルのスタイル設定
        print("セルのスタイル設定")
        set_cell_style(ws, data_info, start_row, headers)

        # ファイルの保存
        wb.save(output_file_path)
        print(f"データが {output_file_path} に保存されました")
        print(separator)

        # 保存したExcelファイルを開く
        subprocess.run(["start", "excel", output_file_path], shell=True)

    except Exception as e:
        print(f"予期せぬエラーが発生しました: {e}")


def add_images(
    ws: Worksheet,
    data_info: list[dict[str, Union[str, int, float]]],
    start_row: int,
) -> None:
    """
    Excelワークシートに画像を追加します。

    Args:
        ws (Worksheet): 画像を追加する対象のExcelワークシート。
        data_info (list[dict[str, Union[str, int, float]]]): 画像追加の対象となる物件情報のリスト。
        start_row (int): データのスタート行。
    """
    image_height_cm = 2.5  # 画像の高さを2.5cmに設定
    cm_to_pixel = 37.7952755906  # cmからピクセルに変換（1cm ≈ 37.7952755906ピクセル）

    for row in range(start_row, start_row + len(data_info)):
        folder_path = data_info[row - start_row].get("フォルダパス", "")
        image_path = os.path.join(str(folder_path), "image.bmp")

        if os.path.exists(image_path):
            img = Image(image_path)
            original_width, original_height = img.width, img.height
            aspect_ratio = original_width / original_height
            image_height_pixels = int(image_height_cm * cm_to_pixel)
            image_width_pixels = int(image_height_pixels * aspect_ratio)

            img.width = image_width_pixels  # 元の縦横比を維持したまま幅を設定
            img.height = image_height_pixels  # 高さを設定
            img.anchor = ws.cell(row=row, column=2).coordinate  # 2列目に画像を配置
            ws.add_image(img)
            ws.row_dimensions[row].height = (
                image_height_cm * 28.3464567
            )  # 行の高さを2.5cmに設定

            # セルの上下中央揃えを設定
            ws.cell(row=row, column=2).alignment = Alignment(
                horizontal="center", vertical="center"
            )

    # セルの幅を画像の幅に合わせる
    ws.column_dimensions["B"].width = (
        image_width_pixels / 7
    )  # ピクセルをExcelのカラム幅単位に変換


def set_hyperlink(
    ws: Worksheet,
    data_info: list[dict[str, Union[str, int, float]]],
    start_row: int,
    headers: list[str],
) -> None:
    """
    Excelワークシートのハイパーリンクを設定します。

    Args:
        ws (Worksheet): ハイパーリンクを設定する対象のExcelワークシート。
        data_info (list[dict[str, Union[str, int, float]]]): ハイパーリンク設定の対象となる物件情報のリスト。
        start_row (int): データのスタート行。
        headers (list[str]): Excelシートのヘッダー行。
    """
    for row in range(start_row, start_row + len(data_info)):
        folder_path = data_info[row - start_row]["フォルダパス"]

        if "物件フォルダ名" in headers:
            folder_name_index = headers.index("物件フォルダ名") + 1
            ws.cell(row=row, column=folder_name_index).hyperlink = os.path.join(
                str(folder_path), "START.ss7"
            )
            ws.cell(row=row, column=folder_name_index).style = "Hyperlink"

        if "フォルダパス" in headers:
            folder_path_index = headers.index("フォルダパス") + 1
            ws.cell(row=row, column=folder_path_index).hyperlink = folder_path
            ws.cell(row=row, column=folder_path_index).style = "Hyperlink"


def set_cell_style(
    ws: Worksheet,
    data_info: list[dict[str, Union[str, int, float]]],
    start_row: int,
    headers: list[str],
) -> None:
    """
    Excelワークシートのセルスタイルを設定します。

    Args:
        ws (Worksheet): スタイルを設定する対象のExcelワークシート。
        data_info (list[dict[str, Union[str, int, float]]]): スタイル設定の対象となる物件情報のリスト。
        start_row (int): データのスタート行。
        headers (list[str]): Excelシートのヘッダー行.
    """
    for row in ws.iter_rows(
        min_row=start_row,
        max_row=start_row + len(data_info) - 1,
        min_col=1,
        max_col=len(headers),
    ):
        for cell in row:
            if isinstance(cell.value, (int, float)):
                cell.alignment = Alignment(
                    horizontal="right", vertical="center"
                )  # 数字は右揃え
            elif isinstance(cell.value, str):
                if cell.value in ["－", "○"]:
                    cell.alignment = Alignment(
                        horizontal="center", vertical="center"
                    )  # 特定の記号や「－」は中央揃え
                else:
                    try:
                        datetime.strptime(
                            cell.value, "%Y/%m/%d %H:%M"
                        )  # 日付形式の場合は右揃え
                        cell.alignment = Alignment(
                            horizontal="right", vertical="center"
                        )
                    except ValueError:
                        cell.alignment = Alignment(
                            horizontal="left", vertical="center"
                        )  # その他の文字列は左揃え
